/****************************************************************************
** COPYRIGHT (C):    1996..1998 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++ 
** FILE:             ccc_ansi.cpp
** PURPOSE:          use if your compiler doesn't fully support ANSI C++
** VERSION           2.0
** PROGRAMMERS:      Cay Horstmann (CSH)
**                   Darrel D. Cherry (DDC)
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
****************************************************************************/

#define EXPORT

#ifndef EXPORT

/* Depending on your compiler, you may need to define the following
   preprocessor variables. For an ANSI C++ compiler, all of them should not
   be defined.

CCC_NEED_STRING                    if your compiler doesn't have a working string type
CCC_NEED_STRINGSTREAM              if your compiler doesn't have working string stream types
CCC_NEED_VECTOR                    if your compiler doesn't have a working vector template
CCC_NEED_LIST                      if your compiler doesn't have a working list template
CCC_USE_OLD_STREAMS                if your compiler doesn't support <iostream>, or it causes conflicts
CCC_USE_OLD_CLIBS                  if your compiler doesn't support <ctime> <cassert> <cmath> <cstdlib>
CCC_USE_OLD_STRSTREA               if your compiler requires <strsrea.h> instead of <strstream.h>
CCC_NEED_TYPENAME                  if your compiler doesn't support template<typename T>
CCC_NEED_STATIC_CAST               if your compiler doesn't support static_cast<T>(x)
CCC_NEED_BOOL                      if your compiler doesn't have the type bool
CCC_NEED_BOOL_OPS                  if your compiler doesn't support the and or not keywords
CCC_DONT_CALL_DESTRUCTORS          if your compiler messes up calls to destructors in templates
CCC_AVOID_UNUSED_ARG_WARNING       if your compiler reports a "arg not used" warning in vector<T>::detach
CCC_UNDEF_MINMAX                   if your compiler defines min and max preprocessor tokens
CCC_NEED_RAND_MAX                   if your compiler doesn't define the RAND_MAX token

*/

#ifdef CCC_USE_ANSI
   #undef CCC_ANSI_H
#else

#if defined(__BORLANDC__)
   #if __BORLANDC__ >= 0x530
      #pragma warn -csu
         /* suppress warning signed/unsigned 
            because vector<T>::size() is unsigned
         */
      #define CCC_NEED_BOOL_OPS
   #else
      #define CCC_USE_OLD_STREAMS
      #define CCC_USE_OLD_CLIBS
      #define CCC_NEED_STRING
      #define CCC_NEED_STRINGSTREAM
      #define CCC_USE_OLD_STRSTREA
      #define CCC_NEED_VECTOR
      #define CCC_NEED_LIST
      #define CCC_UNDEF_MINMAX
      #if __BORLANDC__ >= 0x500
         #define string ccc_string
            // if you don't do that, then something in the Borland startup code
            // calls our string::~string() destructor and the program dies
      #endif
      #define CCC_NEED_BOOL_OPS
         // they don't work right in Borland 5.01--precedence is wrong!!!
      #define CCC_AVOID_UNUSED_ARG_WARNING

      #if __BORLANDC__ < 0x500
         #define CCC_NEED_BOOL
         #define CCC_NEED_TYPENAME
      #endif
      #if __BORLANDC__ < 0x460
         #define CCC_NEED_STATIC_CAST
      #endif
      #if __BORLANDC__ < 0x410
         #define CCC_DONT_CALL_DESTRUCTORS
      #endif
   #endif
#elif defined(__TURBOC__)
   #define CCC_USE_OLD_STREAMS
   #define CCC_USE_OLD_CLIBS
   #define CCC_NEED_STRING
   #define CCC_NEED_STRINGSTREAM
   #define CCC_USE_OLD_STRSTREA
   #define CCC_NEED_VECTOR
   #define CCC_NEED_BOOL_OPS
   #define CCC_AVOID_UNUSED_ARG_WARNING
   #define CCC_NEED_LIST
   #define CCC_UNDEF_MINMAX

   #if __TURBOC__ < 0x500
      #define CCC_NEED_BOOL
      #define CCC_NEED_STATIC_CAST
      #define CCC_NEED_TYPENAME
   #endif
   #if __TURBOC__ < 0x410
      #define CCC_DONT_CALL_DESTRUCTORS
   #endif

#elif defined(__SC__)
   #define CCC_USE_OLD_STREAMS
   #define CCC_USE_OLD_CLIBS
   #define CCC_NEED_STRING
   #define CCC_NEED_STRINGSTREAM
   #define CCC_USE_OLD_STRSTREA
   #define CCC_NEED_VECTOR
   #define CCC_NEED_LIST
   #define CCC_NEED_BOOL
   #define CCC_UNDEF_MINMAX

#elif defined(_MSC_VER) // last because SC defines _MSVC
   #if (_MSC_VER <= 1000) // MSVC 4.0
      #define CCC_NEED_BOOL
   #endif 

   #if (_MSC_VER < 1200)
      #define CCC_DONT_CALL_STATIC_DESTRUCTORS
      #define CCC_NEED_VECTOR
   #endif

   #define CCC_NEED_BOOL_OPS
   #define CCC_UNDEF_MINMAX

#elif defined(__GNUC__)  // for g++
   #define CCC_NEED_BOOL_OPS
   // your Gnu C++ may not need these
   #define CCC_USE_OLD_STREAMS
   #define CCC_USE_OLD_CLIBS
   #define CCC_NEED_STRING
   #define CCC_NEED_STRINGSTREAM
   #define CCC_NEED_VECTOR
   #define CCC_NEED_LIST
   #define CCC_NEED_RAND_MAX

#elif defined(__MWERKS__)
   #define CCC_UNDEF_MINMAX
   #pragma require_prototypes off
#endif


/*-------------------------------------------------------------------------*/

#ifdef CCC_NEED_BOOL
   #define false 0
   #define true 1
   typedef int bool;
   #define CCC_NEED_BOOL_OPS
#endif


#ifdef CCC_NEED_BOOL_OPS
   #define and &&
   #define or ||
   #define not !
#endif


#ifdef CCC_NEED_TYPENAME
   #define typename class
#endif


#ifdef CCC_NEED_STATIC_CAST

/* This is a very rudimentary implementation of static_cast that only
   works for casts between numeric types.
*/

template<class T> class static_cast
{
public:
   static_cast(double x) : _t((T)x) {}
   static_cast(float x) : _t((T)x) {}
   static_cast(long x) : _t((T)x) {}
   static_cast(unsigned long x) : _t((T)x) {}
   static_cast(char x) : _t((T)x) {}
   static_cast(unsigned char x) : _t((T)x) {}
   static_cast(int x) : _t((T)x) {}
   static_cast(unsigned int x) : _t((T)x) {}
   static_cast(short x) : _t((T)x) {}
   static_cast(unsigned short x) : _t((T)x) {}
   operator T() const { return _t; }
private:
   T _t;
};

#endif


#ifdef CCC_UNDEF_MINMAX

#undef min
#undef max

#endif


#ifdef CCC_USE_OLD_STREAMS
   #include <iostream.h>
   #include <iomanip.h>
   #include <fstream.h>
   inline ostream& fixed(ostream& os)
   {  os.setf(ios::fixed, ios::floatfield);
      return os;
   }
#else
   #include <iostream>
   #include <iomanip>
   #include <fstream>
#endif


#ifdef CCC_USE_OLD_CLIBS
   #include <stdlib.h>
   #include <math.h>
   #include <time.h>
   #include <assert.h>
#else
   #include <cstdlib>
   #include <cmath>
   #include <ctime>
   #include <cassert>
#endif


#ifdef CCC_NEED_VECTOR
   #include "ccc_vect.h"
#else
   #include <vector>
   #define CCC_NEED_NAMESPACE_STD
#endif


#ifdef CCC_NEED_STRING
   #include "ccc_strg.h"
#else
   #include <string>
   #define CCC_NEED_NAMESPACE_STD
#endif


#ifdef CCC_NEED_STRINGSTREAM
   #ifdef CCC_USE_OLD_STRSTREA
      #include <strstrea.h>
   #else
      #include <strstream.h>
   #endif

   class ostringstream : public ostrstream
   {
   /* we only need the str() function--it returns a char*, not a string
      because we can't change the return type.
      We rely on the automatic type conversion--it works for the examples
      in the book
   */
   public:
      char* str() { put('\0'); return ostrstream::str(); }
   };

   class istringstream : public istrstream
   {
   public:
      istringstream(const string& s) : istrstream((char*)s.c_str()) {}
   };
#else
   #include <sstream>
   #define CCC_NEED_NAMESPACE_STD
#endif


#ifdef CCC_NEED_LIST
   #include "ccc_list.h"
#else
   #include <list>
   #define CCC_NEED_NAMESPACE_STD
#endif


#ifdef CCC_NEED_NAMESPACE_STD
   using namespace std;
#endif


#ifdef CCC_NEED_RAND_MAX

   #ifndef RAND_MAX
      #define RAND_MAX 0x7FFF
   #endif

#endif


#endif
   // CCC_USE_ANSI

#endif
   // EXPORT

#include "ccc_ansi.h"

#ifdef CCC_NEED_STRING
   #include "ccc_strg.cpp"
#endif

#ifdef CCC_NEED_VECTOR
   #include "ccc_vect.cpp"
#endif

#ifdef CCC_NEED_LIST
   #include "ccc_list.cpp"
#endif


/***************************************************************************/
