/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
**
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You 
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
**
** PROJECT:          Computing Concepts with C++ 
** FILE:             ccc_asc.cpp
** PURPOSE:          Graphics window for BGI graphics
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       2-19-98  (CSH)
** UPDATE HISTORY:
**
****************************************************************************/

/* IMPORT ******************************************************************/

#define EXPORT

#ifndef EXPORT

#include "ccc_ansi.h"

#ifdef CCC_USE_OLD_CLIBS
   #include <math.h>
   #include <stdlib.h>
   #include <string.h>
#else
   #include <cmath>
   #include <cstdlib>
   #include <cstring>
#endif

#ifdef CCC_USE_OLD_STREAMS
   #include <iostream.h>
#else
   #include <iostream>
#endif

#endif
   // EXPORT

#include "ccc_asc.h"

/* CLASSES *****************************************************************/

#ifndef EXPORT

class GraphicWindow
{
public:
   GraphicWindow();
   ~GraphicWindow();

   GraphicWindow& operator<<(Point p);
   GraphicWindow& operator<<(Circle c);
   GraphicWindow& operator<<(Line s);
   GraphicWindow& operator<<(Message t);

   string get_string(string prompt);
   double get_double(const string& prompt) { return atof(get_string(prompt).c_str()); }
   int get_int(const string& prompt) { return atoi(get_string(prompt).c_str()); }
   Point get_mouse(string prompt);

   void coord(double xmin, double ymin, double xmax, double ymax);
   void clear();

   void flush();
private:
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   double user_to_disp_dx(double x) const;
   double user_to_disp_dy(double y) const;

   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;
   double disp_to_user_x(int x) const;
   double disp_to_user_y(int y) const;

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void ellipse(double x, double y, double ra, double rb);
   void text(string t, double x, double y);
   
   void set(int x, int y);
   void set(int x, int y, char ch);
   void set4(int x, int y, int cx, int cy);

   
   GraphicWindow::GraphicWindow(const GraphicWindow&);
   GraphicWindow& operator=(const GraphicWindow&);

   double _user_xmin;
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;

   int _disp_xmax;
   int _disp_ymax;
   int _disp_xoff;
   int _disp_yoff;

   enum { XSIZE = 60, YSIZE = 24, XMAXSIZE = 79 };
   /* graphics clipped at 60 (because 60 * 24 is a square region), 
      text clipped at 79
   */

   char g[YSIZE][XMAXSIZE+1];
   bool dirty;
};

#endif
   // EXPORT

/* GLOBALS *****************************************************************/

EXPORT GraphicWindow cwin;

/* FUNCTIONS ***************************************************************/

Point GraphicWindow::get_mouse(string s)
{  cout << s << " [x y] ";
   string sx;
   string sy;
   cin >> sx >> sy;
   return Point(atof(sx.c_str()), atof(sy.c_str()));
}

//............................................................................

string GraphicWindow::get_string(string s)
{  flush();
   cout << s;
   string r;
   getline(cin, r);
   return r;
}

//............................................................................

GraphicWindow::GraphicWindow()
{  const double DEFAULT_XMIN = -10;
   const double DEFAULT_YMIN = 10;
   const double DEFAULT_XMAX = 10;
   const double DEFAULT_YMAX = -10;

   _disp_xoff = 0; 
   _disp_yoff = 0;
   _disp_xmax = XSIZE;
   _disp_ymax = YSIZE;

   clear();
   coord(DEFAULT_XMIN, DEFAULT_YMIN, DEFAULT_XMAX, DEFAULT_YMAX);
}

//............................................................................

GraphicWindow::~GraphicWindow()
{  flush();
}

//............................................................................

void GraphicWindow::flush()
{  if (!dirty) return;
   for (int y = 0; y < YSIZE; y++)
      cout << g[y] << "\n";
   dirty = false;
}

//............................................................................

void GraphicWindow::coord(double xmin, double ymin,
      double xmax, double ymax)
/* PURPOSE:    Set the logical coordinate system
   RECEIVES:   xmin, ymin - coordinates of the top left corner
               xmax, ymax - coordinates of the bottom right corner
*/
{  _user_xmin = xmin;
   _user_xmax = xmax;
   _user_ymin = ymin;
   _user_ymax = ymax;
}

//............................................................................

int GraphicWindow::user_to_disp_x(double x) const
/* PURPOSE:    scales the x coordinate from user to display coordinates
   RECEIVES:   x - a coordinate position in user coordinates
   RETURNS:    the scaled coordinate
*/
{  return (int)((x - _user_xmin) * _disp_xmax / (_user_xmax - _user_xmin));
}

//............................................................................

int GraphicWindow::user_to_disp_y(double y) const
/* PURPOSE:    scales the y coordinate from user to display coordinates
   RECEIVES:   y - a coordinate position in user coordinates
   RETURNS:    the scaled coordinate
*/
{  return (int)((y - _user_ymin) * _disp_ymax / (_user_ymax - _user_ymin));
}

//............................................................................

double GraphicWindow::user_to_disp_dx(double x) const
/* PURPOSE:    scales an x distance from user to display coordinates
   RECEIVES:   x - a distance in user coordinates
   RETURNS:    the scaled distance
*/
{  return (x * _disp_xmax) / (_user_xmax - _user_xmin);
}

//............................................................................

double GraphicWindow::user_to_disp_dy(double y) const
/* PURPOSE:    scales an y distance from user to display coordinates
   RECEIVES:   y - a distance in user coordinates
   RETURNS:    the scaled distance
*/
{  return (y * _disp_ymax) / (_user_ymax - _user_ymin);
}

//............................................................................

double GraphicWindow::disp_to_user_dx(int x) const
/* PURPOSE:    scales an x distance from display to user coordinates
   RETURNS:    the scaled distance
*/
{  return x * (_user_xmax - _user_xmin) / _disp_xmax;
}

//............................................................................

double GraphicWindow::disp_to_user_dy(int y) const
/* PURPOSE:    scales an y distance from user to display coordinates
   RETURNS:    the scaled distance
*/
{  return y * (_user_ymax - _user_ymin) / _disp_ymax;
}

//............................................................................

double GraphicWindow::disp_to_user_x(int x) const
/* PURPOSE:    scales an x distance from display to user coordinates
   RETURNS:    the scaled distance
*/
{  return (x - _disp_xoff) * (_user_xmax - _user_xmin) / _disp_xmax + _user_xmin;
}

//............................................................................

double GraphicWindow::disp_to_user_y(int y) const
/* PURPOSE:    scales an y distance from user to display coordinates
   RETURNS:    the scaled distance
*/
{  return (y - _disp_yoff) * (_user_ymax - _user_ymin) / _disp_ymax + _user_ymin;
}

//............................................................................

void GraphicWindow::set(int x, int y)
{  if (0 <= x && x <= XSIZE && 0 <= y && y < YSIZE)
      g[y][x] = '*';
}

//............................................................................

void GraphicWindow::set(int x, int y, char ch)
{  if (0 <= x && x < XMAXSIZE && 0 <= y && y < YSIZE)
      g[y][x] = ch;
}

//............................................................................

void GraphicWindow::point(double x, double y)
/* PURPOSE:    plot a point
   RECEIVES:   x, y - the point coordinates
*/
{  int cx = user_to_disp_x(x);
   int cy = user_to_disp_y(y);
   
   set(cx, cy);
   dirty = true;
}

//............................................................................

void GraphicWindow::line(double xfrom, double yfrom, double xto, double yto)
/* PURPOSE:    plot a line segment
   RECEIVES:   xfrom, yfrom - starting point
               xto, yto - ending point
*/
{  int x1 = user_to_disp_x(xfrom);
   int x2 = user_to_disp_x(xto);
   int y1 = user_to_disp_y(yfrom);
   int y2 = user_to_disp_y(yto);
   
   int dx = x2 - x1;
   int dy = y2 - y1;
   if (abs(dy) < abs(dx)) // |slope| < 1
   {  if (dx < 0) 
      {  int temp = x1; x1 = x2; x2 = temp;
         temp = y1; y1 = y2; y2 = temp;
         dx = -dx; dy = -dy;
      }
      double y = y1 + 0.5;
      for (int x = x1; x != x2; x++)
      {  set(x, int(y));
         y += double(dy) / double(dx);
      }
   }
   else
   {  if (dy < 0) 
      {  int temp = x1; x1 = x2; x2 = temp;
         temp = y1; y1 = y2; y2 = temp;
         dx = -dx; dy = -dy;
      }
      double x = x1 + 0.5;
      for (int y = y1; y != y2; y++)
      {  set(int(x), y);
         x += double(dx) / double(dy);
      }
   }
   set(x2, y2);

   dirty = true;
}

//............................................................................

void GraphicWindow::clear()
/* PURPOSE:    Clear the display
*/
{  for (int y = 0; y < YSIZE; y++)
   {  for (int x = 0; x < XMAXSIZE; x++)
         g[y][x] = ' ';
      g[y][XMAXSIZE] = '\0';
   }
   dirty = false;
}

//............................................................................

void GraphicWindow::set4(int x, int y, int cx, int cy)
{  set(cx + x, cy + y);
   set(cx + x, cy - y);
   set(cx - x, cy + y);
   set(cx - x, cy - y);
}

//............................................................................

void GraphicWindow::ellipse(double center_x, double center_y, double ra, double rb)
/* PURPOSE:    Plot an ellipse
   RECEIVES:   x, y - center
               ra, rb - radii in x- and y- direction
   REMARKS:    Foley, van Dam, Feiner & Hughes p. 90
*/

{  int cx = user_to_disp_x(center_x);
   int cy = user_to_disp_y(center_y);
   double a = fabs(user_to_disp_dx(ra));
   double b = fabs(user_to_disp_dy(rb));
   
   int x = 0;
   int y = (int)(b + 0.5);
   double d1 = b * b - a * a * b + a * a / 4;
   set4(x, y, cx, cy);
   
   while (a * a * (y - 0.5) > b * b * (x + 1))
   {  if (d1 < 0)
      {  d1 += b * b * (2 * x + 3);
         x++;
      }
      else
      {  d1 += b * b * (2 * x + 3) + a * a * (-2 * y + 2);
         x++;
         y--;
      }
      set4(x, y, cx, cy);
   }
   
   double d2 = b * b * (x + 0.5) * (x + 0.5) + a * a * (y - 1) * (y - 1) - a * a * b * b;
   while (y > 0)
   {  if (d2 < 0)
      {  d2 += b * b * (2 * x + 2) + a * a * (-2 * y + 3);
         x++;
         y--;
      }
      else
      {  d2 += a * a * (-2 * y + 3);
         y--;
      }
      set4(x, y, cx, cy);
   }

   dirty = true;
}

//............................................................................

void GraphicWindow::text(string t, double x, double y)
/* PURPOSE:    plot text in the current font
   RECEIVES:   t - the text
               x, y - the top left corner
*/
{  int dx = user_to_disp_x(x);
   int dy = user_to_disp_y(y);
   for (int i = 0; i < t.length(); i++) set(dx + i, dy, t[i]);
   dirty = true;
}

//............................................................................

GraphicWindow& GraphicWindow::operator<<(Line s)
{  line(s.get_start().get_x(), s.get_start().get_y(),
         s.get_end().get_x(), s.get_end().get_y());
   return *this;
}

//............................................................................

GraphicWindow& GraphicWindow::operator<<(Message t)
{
   text(t.get_message(), t.get_start().get_x(), t.get_start().get_y());
   return *this;
}

//............................................................................//............................................................................//............................................................................

GraphicWindow& GraphicWindow::operator<<(Circle c)
{
   ellipse(c.get_center().get_x(), c.get_center().get_y(),
         c.get_radius(), c.get_radius());
   return *this;
}

//............................................................................//............................................................................//............................................................................

GraphicWindow& GraphicWindow::operator<<(Point p)
{  point (p.get_x(), p.get_y());
   return *this;
}

