#ifndef CCC_ASC_H
#define CCC_ASC_H

/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
**
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You 
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
**
** PROJECT:          Computing Concepts with C++ 
** FILE:             ccc_asc.h
** PURPOSE:          Graphics window for BGI graphics
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       2-19-98  (CSH)
** UPDATE HISTORY:
**
****************************************************************************/

#line 22 "ccc_asc.cpp"


#include "ccc_ansi.h"

#ifdef CCC_USE_OLD_CLIBS
   #include <math.h>
   #include <stdlib.h>
   #include <string.h>
#else
   #include <cmath>
   #include <cstdlib>
   #include <cstring>
#endif

#ifdef CCC_USE_OLD_STREAMS
   #include <iostream.h>
#else
   #include <iostream>
#endif


#line 49 "ccc_asc.cpp"


class GraphicWindow
{
public:
   GraphicWindow();
   ~GraphicWindow();

   GraphicWindow& operator<<(Point p);
   GraphicWindow& operator<<(Circle c);
   GraphicWindow& operator<<(Line s);
   GraphicWindow& operator<<(Message t);

   string get_string(string prompt);
   double get_double(const string& prompt) { return atof(get_string(prompt).c_str()); }
   int get_int(const string& prompt) { return atoi(get_string(prompt).c_str()); }
   Point get_mouse(string prompt);

   void coord(double xmin, double ymin, double xmax, double ymax);
   void clear();

   void flush();
private:
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   double user_to_disp_dx(double x) const;
   double user_to_disp_dy(double y) const;

   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;
   double disp_to_user_x(int x) const;
   double disp_to_user_y(int y) const;

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void ellipse(double x, double y, double ra, double rb);
   void text(string t, double x, double y);
   
   void set(int x, int y);
   void set(int x, int y, char ch);
   void set4(int x, int y, int cx, int cy);

   
   GraphicWindow::GraphicWindow(const GraphicWindow&);
   GraphicWindow& operator=(const GraphicWindow&);

   double _user_xmin;
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;

   int _disp_xmax;
   int _disp_ymax;
   int _disp_xoff;
   int _disp_yoff;

   enum { XSIZE = 60, YSIZE = 24, XMAXSIZE = 79 };
   /* graphics clipped at 60 (because 60 * 24 is a square region), 
      text clipped at 79
   */

   char g[YSIZE][XMAXSIZE+1];
   bool dirty;
};


#line 119 "ccc_asc.cpp"
extern GraphicWindow cwin;

#endif
