#ifndef CCC_BGI_H
#define CCC_BGI_H

/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
**
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You 
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
**
** PROJECT:          Computing Concepts with C++ 
** FILE:             ccc_bgi.h
** PURPOSE:          Graphics window for BGI graphics
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
**                   Jiaoyang Zhou
** START DATE:       2-19-94  (CSH)
** UPDATE HISTORY:
**
****************************************************************************/

#line 23 "ccc_bgi.cpp"


#ifdef CCC_USE_OLD_CLIBS
   #include <math.h>
   #include <string.h>
   #include <stdlib.h>
   #include <stdio.h>
#else
   #include <cmath>
   #include <cstring>
   #include <cstdlib>
   #include <cstdio>
#endif

#include <dos.h>
#include <bios.h>
#include <graphics.h>

#include <io.h>
#include <conio.h>


#line 51 "ccc_bgi.cpp"


class GraphicWindow;

class MouseEvent
{
public:
   enum Type { NOT_INSTALLED, LBUTTONDOWN, MOUSEMOVE, LBUTTONUP };
   MouseEvent(Type, double, double);

   Type type() const { return _type; }
   double x() const { return _x; }
   double y() const { return _y; }
private:
   Type _type;
   double _x;
   double _y;
};

/*-------------------------------------------------------------------------*/

class Mouse
{
public:
   Mouse(GraphicWindow& w);

   MouseEvent read(const GraphicWindow& gc);

   bool is_installed() const;

   int get_position(int&, int&) const;
   void show_cursor();
   void hide_cursor();
private:
   enum
   {  MOUSE_INTR = 0x33,
      RETF = 0xCF, // far return instruction
      SHOWCURSOR = 1,
      HIDECURSOR = 2,
      READMOUSE = 3,
   };
   bool _installed;
   GraphicWindow* ws;
};

/*-------------------------------------------------------------------------*/

class GraphicWindow
{
public:
   GraphicWindow();
   ~GraphicWindow();

   GraphicWindow& operator<<(Point p);
   GraphicWindow& operator<<(Circle c);
   GraphicWindow& operator<<(Line s);
   GraphicWindow& operator<<(Message t);

   string get_string(string prompt);
   int get_int(const string& out_string) 
   {  return atoi(get_string(out_string).c_str()); 
   }
   double get_double(const string& out_string) 
   {  return atof(get_string(out_string).c_str()); 
   }
   Point get_mouse(string prompt);

   void coord(double xmin, double ymin, double xmax, double ymax);
   void clear();

private:
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   int user_to_disp_dx(double x) const;
   int user_to_disp_dy(double y) const;

   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;
   double disp_to_user_x(int x) const;
   double disp_to_user_y(int y) const;

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void ellipse(double x, double y, double ra, double rb);

   void text(string t, double x, double y);

   void statusline_prompt(string s);
   void statusline_echo(string s);
   void statusline_clear();

   GraphicWindow::GraphicWindow(const GraphicWindow&);
   GraphicWindow& operator=(const GraphicWindow&);

   enum { POINT_RADIUS = 2 };

   double _user_xmin;
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;

   int _disp_xmax;
   int _disp_ymax;
   int _disp_xoff;
   int _disp_yoff;

   Mouse _mouse;
   string _prompt;   // current string in the graphics window
   string _echo;

friend class Mouse;
};


#line 169 "ccc_bgi.cpp"
extern GraphicWindow cwin;

#endif
