/****************************************************************************
** COPYRIGHT (C):    1994..1998 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++ 2E
** FILE:             ccc_msw.cpp
** PURPOSE:          Graphics window class (Microsoft Windows version)
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
**                   Charles Miller (CBM)
**                   Darrel D. Cherry (DDC)
**                   Jiaoyang Zhou (JYZ)
**                   Geof Pawlicki (GFP)
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
****************************************************************************/

/* IMPORT ******************************************************************/

#define EXPORT

#ifndef EXPORT

#ifdef CCC_USE_OLD_CLIBS
   #include <math.h>
   #include <string.h>
   #include <stdlib.h>
   #include <stdio.h>
#else
   #include <cmath>
   #include <cstring>
   #include <cstdlib>
   #include <cstdio>
#endif

#include <windows.h>

// avoid conflicts with Windows function names
#define Polygon CCC_WIN_Polygon
#define Ellipse CCC_WIN_Ellipse
#define Rectangle CCC_WIN_Rectangle

#endif
   // EXPORT

#include "ccc_msw.h"

/* CONSTANTS ***************************************************************/

const double DEFAULT_XMIN = -10;
const double DEFAULT_YMIN = 10;
const double DEFAULT_XMAX = 10;
const double DEFAULT_YMAX = -10;

/* TYPES *******************************************************************/

#ifndef EXPORT

class GraphicBitmap
{
public:
   GraphicBitmap();
   void init(HWND hwnd);
   void close() const;
   void repaint(PAINTSTRUCT&) const;
   void clear() const;
   HDC hdc() const;

private:
   HDC _hdc; // a long-lived device context
   HBITMAP _hbm, _hbm_saved; // handles to the created bitmap objects
   int _disp_xmax; // width of the bitmap (in bits)
   int _disp_ymax; // height of the bitmap (in bits)
};

/*-------------------------------------------------------------------------*/

class GraphicWindow
// PURPOSE: Sends graphics primitives to a display window
{
public:
   GraphicWindow();
   void open(HWND hwnd, HDC mainwin_hdc);
   void coord(double xmin, double ymin, double xmax, double ymax);
   void clear();

   GraphicWindow& operator<<(Point p);
   GraphicWindow& operator<<(Circle c);
   GraphicWindow& operator<<(Line s);
   GraphicWindow& operator<<(Message t);

   string get_string(string out_string);
   int get_int(const string& out_string) 
   {  return atoi(get_string(out_string).c_str()); 
   }
   double get_double(const string& out_string) 
   {  return atof(get_string(out_string).c_str()); 
   }
   Point get_mouse(string out_string);

private:
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   double disp_to_user_x(int x) const;
   double disp_to_user_y(int y) const;

   void statusline_prompt(string s);

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void ellipse(double x, double y, double ra, double rb);

   void text(string t, double x, double y);

   double _user_xmin; // the user's window's logical window coords
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;
   int _disp_xmax; // the physical window dimension (in pixels)
   int _disp_ymax;

   GraphicBitmap& _bm; // the bitmap to draw on
   HDC _hdc; // device context to the display
   int _mlength; // length of text extent for message child window
   int _elength; // length of text extent for edit child window
};

#endif
   // EXPORT

/* GLOBALS *****************************************************************/

static GraphicBitmap mainwin_bm;

static HWND child_hwnd;
static HWND edit_hwnd;

EXPORT GraphicWindow cwin;

/* FUNCTIONS ***************************************************************/

int ccc_win_main();

/*-------------------------------------------------------------------------*/

static 
/* CSH 1998-06-10 required by Metrowerks to avoid idiotic warning 
   about function with no prototype
*/
long FAR PASCAL
#if !(defined(_WIN32) || defined(__WIN32__))
_export
#endif
ccc_win_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
/* PURPOSE:    Window procedure for graphics window
*/
{  static int paint_flag = 1;   // tells WinProc to call main()
   PAINTSTRUCT ps;            // the display's paint struct
   HDC mainwin_hdc;
   HINSTANCE hInstance;
  
   switch (message)
   {  case WM_CREATE:
         hInstance = (HINSTANCE)
//gfp - BCC wants __WIN32__
//      MSVC wants _WIN32
#if defined(_WIN32) || defined(__WIN32__)
         GetWindowLong(hwnd, GWL_HINSTANCE);
#else
         GetWindowWord(hwnd, GWW_HINSTANCE);
#endif

         child_hwnd = CreateWindow("static",
            NULL,
            WS_CHILD | WS_VISIBLE, /* style */
            0,  /* x position */
            0,  /* y position */
            0,  /* width (x) */
            30, /* height (y) */
            hwnd,
            (HMENU)84,  /* child ID */
            hInstance,
            0);

         edit_hwnd  = CreateWindow("edit",
            NULL,
            WS_CHILD | WS_VISIBLE | ES_MULTILINE | ES_AUTOVSCROLL, /* style */
            0,  /* x position */
            0,  /* y position */
            0,  /* width (x) */
            30, /* height (y) */
            hwnd,
            (HMENU)85,  /* child ID */
            hInstance,
            0);

         return 0;

      case WM_PAINT:
         mainwin_hdc = BeginPaint(hwnd, &ps);

         if (paint_flag)
         {  paint_flag = false;       // flag must be set first in case
            mainwin_bm.init(hwnd);    // of multiple calls.
            cwin.open(hwnd, mainwin_hdc);
            ccc_win_main();
         }
         else
            mainwin_bm.repaint(ps);

         EndPaint(hwnd, &ps);
         break;

      case WM_DESTROY:
         mainwin_bm.close();
         // PostQuitMessage(0);
         exit(0); 
         /* CSH 1-28-98 otherwise program won't die 
            when in secondary msg loop
         */
         break;
   }
   return DefWindowProc(hwnd, message, wParam, lParam);
}

/*-------------------------------------------------------------------------*/

//gfp - different args to WinMain if WIN32
int PASCAL WinMain(
#if defined(_WIN32) || (__WIN32__)
   HINSTANCE hInstance,
   HINSTANCE hPrevInstance,
#else
   HANDLE hInstance,
   HANDLE hPrevInstance,
#endif
   LPSTR,
   int nShowCmd)
{  MSG msg;
   WNDCLASS wndclass;

   if (!hPrevInstance)
   {  wndclass.style = CS_HREDRAW | CS_VREDRAW;
      wndclass.lpfnWndProc = ccc_win_proc;
      wndclass.cbClsExtra = 0;
      wndclass.cbWndExtra = 0;
      wndclass.hInstance = hInstance;
      wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
      wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
      wndclass.hbrBackground = (HBRUSH)GetStockObject (WHITE_BRUSH);
      wndclass.lpszMenuName = NULL;
      wndclass.lpszClassName = "CCC_WIN";

      RegisterClass (&wndclass);
   }

   char title[80];
   GetModuleFileName(hInstance, title, sizeof(title));

   HWND hwnd = CreateWindow("CCC_WIN",
      title,
      WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX,
      CW_USEDEFAULT,
      CW_USEDEFAULT,
      GetSystemMetrics(SM_CYFULLSCREEN) * 3 / 4,
      GetSystemMetrics(SM_CYFULLSCREEN) * 3 / 4,
      NULL,
      NULL,
      hInstance,
      0);

   ShowWindow(hwnd, nShowCmd);
   UpdateWindow(hwnd);

   while (GetMessage(&msg, NULL, 0, 0))
   {  TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return msg.wParam;
}

/*-------------------------------------------------------------------------*/

GraphicWindow::GraphicWindow()
:  _user_xmin(DEFAULT_XMIN),
   _user_ymin(DEFAULT_YMIN),
   _user_xmax(DEFAULT_XMAX),
   _user_ymax(DEFAULT_YMAX),
   _bm(mainwin_bm)
{}

void GraphicWindow::open(HWND hwnd, HDC mainwin_hdc)
{  RECT rect;
   GetClientRect(hwnd, &rect);
   _disp_xmax = rect.right - 1;
   _disp_ymax = rect.bottom - 1;

   _hdc = mainwin_hdc;

   SelectObject(_hdc, GetStockObject(NULL_BRUSH));
   SelectObject(_bm.hdc(), GetStockObject(NULL_BRUSH));

   SelectObject(_hdc, GetStockObject(BLACK_PEN));
   SelectObject(_bm.hdc(), GetStockObject(BLACK_PEN));

   SelectObject(_hdc, GetStockObject(SYSTEM_FONT));
   SelectObject(_bm.hdc(), GetStockObject(SYSTEM_FONT));

   clear();
}

//...........................................................................

void GraphicWindow::clear()
/* PURPOSE:    Clears the DC the gc is in, and the bitmap too.
*/
{  COLORREF color = RGB(255, 255, 255);
   HBRUSH brush = CreateSolidBrush(color);
   HBRUSH saved_brush = (HBRUSH)SelectObject(_hdc, brush);
   PatBlt(_hdc, 0, 0, _disp_xmax, _disp_ymax, PATCOPY);
   SelectObject(_hdc, saved_brush);
   DeleteObject(brush);
   _bm.clear();
}

//...........................................................................

void GraphicWindow::coord(double xmin, double ymin,
   double xmax, double ymax)
/* PURPOSE:    Set the logical coordinate system
   RECEIVES:   xmin, ymin - coordinates of the top left corner
               xmax, ymax - coordinates of the bottom right corner
*/
{  _user_xmin = xmin;
   _user_xmax = xmax;
   _user_ymin = ymin;
   _user_ymax = ymax;
}

//...........................................................................

int GraphicWindow::user_to_disp_x(double x) const
/* PURPOSE:    scales the x coordinate from user to display coordinates
   RECEIVES:   x - a coordinate position in user coordinates
   RETURNS:    the scaled coordinate
*/
{  return (int) ((x - _user_xmin) * _disp_xmax / (_user_xmax - _user_xmin));
}

//...........................................................................

int GraphicWindow::user_to_disp_y(double y) const
/* PURPOSE:    scales the y coordinate from user to display coordinates
   RECEIVES:   y - a coordinate position in user coordinates
   RETURNS:    the scaled coordinate
*/
{  return (int) ((y - _user_ymin) * _disp_ymax / (_user_ymax - _user_ymin));
}

//...........................................................................

double GraphicWindow::disp_to_user_x(int x) const
/* PURPOSE:    scales an x distance from display to user coordinates
   RETURNS:    the scaled distance
*/

{  return (double)x * (_user_xmax - _user_xmin) / _disp_xmax + _user_xmin;
}


//...........................................................................

double GraphicWindow::disp_to_user_y(int y) const
/* PURPOSE:    scales an y distance from display to user coordinates
   RETURNS:    the scaled distance
*/
{  return (double)y * (_user_ymax - _user_ymin) / _disp_ymax + _user_ymin;
}


//...........................................................................

void GraphicWindow::point(double x, double y)
/* PURPOSE:    plot a point
   RECEIVES:   x, y - the point coordinates
*/
{  const int POINT_RADIUS = 3;
   int disp_x = user_to_disp_x(x);
   int disp_y = user_to_disp_y(y);

#undef Ellipse
   Ellipse(_hdc, disp_x - POINT_RADIUS, disp_y - POINT_RADIUS,
      disp_x + POINT_RADIUS, disp_y + POINT_RADIUS);
   Ellipse(_bm.hdc(), disp_x - POINT_RADIUS, disp_y - POINT_RADIUS,
      disp_x + POINT_RADIUS, disp_y + POINT_RADIUS);
#define Ellipse CCC_WIN_Ellipse
}

//...........................................................................

void GraphicWindow::ellipse(double x, double y, double ra, double rb)
/* PURPOSE:    Plot an ellipse on device context _hdc
   RECEIVES:   x, y - center
               ra, rb - radii in x- and y- direction
*/
{
#undef Ellipse
   Ellipse(_hdc, user_to_disp_x(x - ra), user_to_disp_y(y - rb),
      user_to_disp_x(x + ra),user_to_disp_y(y + rb));
   Ellipse(_bm.hdc(), user_to_disp_x(x - ra), user_to_disp_y(y - rb),
      user_to_disp_x(x + ra),user_to_disp_y(y + rb));
#define Ellipse CCC_WIN_Ellipse
}

//...........................................................................

void GraphicWindow::line(double xfrom, double yfrom, double xto,
   double yto)
/* PURPOSE:    plot a line segment with current pen style and color
   RECEIVES:   _hdc - the DC on which to draw
               xfrom, yfrom - starting point
               xto, yto - ending point
*/
{
//gfp - __WIN32__ for BCC5.01, _WIN32 for MSVC4.0
#if defined(_WIN32) || defined(__WIN32__)
   MoveToEx(_hdc, user_to_disp_x(xfrom), user_to_disp_y(yfrom), 0);
#else
   MoveTo(_hdc, user_to_disp_x(xfrom), user_to_disp_y(yfrom));
#endif
   LineTo(_hdc,user_to_disp_x(xto), user_to_disp_y(yto));

//gfp - __WIN32__ for BCC5.01, _WIN32 for MSVC4.0
#if defined(_WIN32) || defined(__WIN32__)
   MoveToEx(_bm.hdc(), user_to_disp_x(xfrom), user_to_disp_y(yfrom), 0);
#else
   MoveTo(_bm.hdc(), user_to_disp_x(xfrom), user_to_disp_y(yfrom));
#endif
   LineTo(_bm.hdc(),user_to_disp_x(xto), user_to_disp_y(yto));
}

//...........................................................................

void GraphicWindow::text(string s, double x, double y)
/* PURPOSE:    plot text in the current font
   RECEIVES:   t - the text
               x, y - top left coordinate
*/
{  const char* t = s.c_str();
   SetBkMode(_hdc, TRANSPARENT);
   TextOut(_hdc, user_to_disp_x(x), user_to_disp_y(y), t, lstrlen(t));
   SetBkMode(_bm.hdc(), TRANSPARENT);
   TextOut(_bm.hdc(), user_to_disp_x(x), user_to_disp_y(y), t, lstrlen(t));
}

//...........................................................................

void GraphicWindow::statusline_prompt(string s)
/* PURPOSE:    outputs a string to the static child window
        RECEIVES:   s - the string
*/
{  const int M_WIDTH = 30;
   const char* t = s.c_str();
   HDC child_hdc;

   child_hdc = GetDC(child_hwnd);
//gfp -  __WIN32__ for BCC5.01, _WIN32 for MSVC4.0
#if defined (_WIN32) || defined(__WIN32__)
   SIZE sz;
   GetTextExtentPoint32(child_hdc, t, strlen(t), &sz);
   _mlength = sz.cx;
   GetTextExtentPoint32(child_hdc, " ", 1, &sz);
   _mlength += sz.cx;
#else
   _mlength = LOWORD (GetTextExtent(child_hdc, t, strlen(t)))
                + LOWORD(GetTextExtent(child_hdc, " ", 1));
#endif
   ReleaseDC(child_hwnd, child_hdc);
   MoveWindow (child_hwnd, 1, 0, _mlength, M_WIDTH, true);
   SendMessage (child_hwnd, WM_SETTEXT,0, (LPARAM) (LPCSTR) t);

   //move edit window from end of child window to edge of main window
   MoveWindow (edit_hwnd, cwin._mlength+1, 0, cwin._disp_xmax, M_WIDTH, true);
}

//...........................................................................

GraphicWindow& GraphicWindow::operator<<(Point p)
/* PURPOSE:    outputs a point to the main window
   RECEIVES:   p - the point
   RETURNS:    a reference to itself (for nested output)
*/
{  point(p.get_x(), p.get_y());
   return *this;
}

//...........................................................................


GraphicWindow& GraphicWindow::operator<<(Circle c)
/* PURPOSE:    outputs a circle to the main window
   RECEIVES:   c - the circle
   RETURNS:    a reference to itself (for nested output)
*/
{  ellipse(c.get_center().get_x(), c.get_center().get_y(), c.get_radius(), c.get_radius());
   return *this;
}

//...........................................................................

GraphicWindow& GraphicWindow::operator<<(Line s)
/* PURPOSE:    outputs a segment to the main window
   RECEIVES:   s - the segment
   RETURNS:    a reference to itself (for nested output)
*/
{  line(s.get_start().get_x(), s.get_start().get_y(), s.get_end().get_x(), s.get_end().get_y());
   return *this;
}

//...........................................................................

GraphicWindow& GraphicWindow::operator<<(Message t)
/* PURPOSE:    outputs text to the main window
   RECEIVES:   t - the text
   RETURNS:    a reference to itself (for nested output)
*/
{  text(t.get_message(), t.get_start().get_x(), t.get_start().get_y());
   return *this;
}

/*-------------------------------------------------------------------------*/

GraphicBitmap::GraphicBitmap()
{  _disp_xmax = GetSystemMetrics(SM_CXFULLSCREEN);
   _disp_ymax = GetSystemMetrics(SM_CYFULLSCREEN);
}

//............................................................................

void GraphicBitmap::init(HWND hwnd)
/* PURPOSE:    Initializes the bitmap window structures.
   RECEIVES:   Handle to the window in which it is created.
   RETURNS:    A device context specific to this bitmap.
   REMARKS:    We need to know a HDC of the main window in order to assure
               the bitmap has the proper number of color planes, bits
               per pixel, etc.
*/
{  HDC hdc = GetDC(hwnd);
   _hdc = CreateCompatibleDC(hdc);
   _hbm = CreateCompatibleBitmap(hdc, _disp_xmax, _disp_ymax);
   ReleaseDC(hwnd, hdc);
   if (!_hbm)
   {  MessageBox(hwnd, (LPSTR)"Memory problems in Bitmap Creation",
         (LPSTR)"ChiLib Error", MB_ICONHAND);
         SendMessage(hwnd, WM_DESTROY, 0, 0L);
         return;
   }
   _hbm_saved = (HBITMAP)SelectObject(_hdc, _hbm);
}


//............................................................................

void GraphicBitmap::close() const
/* PURPOSE:    Deallocates resources for this bitmap
*/
{  SelectObject(_hdc, _hbm_saved);
   DeleteDC(_hdc);
   DeleteObject(_hbm);
}

//............................................................................

void GraphicBitmap::repaint(PAINTSTRUCT& ps) const
/* PURPOSE:    Repaints with the saved bitmap--use when resizing.
   RECEIVES:   A pointer to the PAINTSTRUCT in WinProc
*/
{  BitBlt(ps.hdc, ps.rcPaint.left, ps.rcPaint.top,
      ps.rcPaint.right, ps.rcPaint.bottom, _hdc,
      ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);
}

//............................................................................

void GraphicBitmap::clear() const
/* PURPOSE:    Clears the bitmap
   RECEIVES:   color - the color to fill the bitmap
*/
{  COLORREF color = RGB(255, 255, 255);
   HBRUSH brush = CreateSolidBrush(color);
   HBRUSH saved_brush = (HBRUSH)SelectObject(_hdc, brush);
   PatBlt(_hdc, 0, 0, _disp_xmax, _disp_ymax, PATCOPY);
   SelectObject(_hdc, saved_brush);
   DeleteObject(brush);
   // note our greed in not releasing a DC
}

//...........................................................................

HDC GraphicBitmap::hdc() const
/* RETURNS:    the device context handle associated with this bitmap
*/
{  return _hdc;
}

/*-------------------------------------------------------------------------*/

string GraphicWindow::get_string(string out_string)
/* PURPOSE:    read text from the edit window
   RECEIVES:   out_string - the text to prompt the user
   RETURNS:    the string read in from the edit window
*/
{  MSG msg;
   const int BUFFSIZE = 80;

   char buffer[BUFFSIZE];
   memset(buffer, 0, sizeof(buffer));
   SendMessage (edit_hwnd, WM_SETTEXT,0, (LPARAM) (LPCSTR) buffer);

   string temp;

   statusline_prompt(out_string);   // output prompt

   SetFocus(edit_hwnd);

   HWND phwnd = GetParent(child_hwnd);

   while (GetMessage(&msg, (HWND) NULL, 0, 0))
   {  TranslateMessage(&msg);
      DispatchMessage(&msg);

      switch(msg.message)
      {  case WM_KEYUP:
            if ((msg.wParam == VK_RETURN) || (msg.wParam == VK_ESCAPE))
            {  SendMessage (edit_hwnd, WM_GETTEXT,sizeof(buffer), (LPARAM) (LPCSTR) buffer);
               int buflen = strlen(buffer);
               if (buflen >= 2 && buffer[buflen - 2] == '\r') buffer[buflen - 2] = 0;
               temp = buffer;
               //shrink child windows after use

               MoveWindow (child_hwnd, 0, 0, 0, 0, true);
               MoveWindow (edit_hwnd, 0, 0, 0, 0, true);
               SetFocus(phwnd);
               return temp;
            }
            break;
      }
   }
   return temp; 
}

//...........................................................................

Point GraphicWindow::get_mouse(string outstr)
{  MSG msg;

   Point p;// = new point;
   int mouse_x, mouse_y;

   statusline_prompt(outstr);
   char buf[100];


   while (GetMessage(&msg, (HWND) NULL, 0, 0))
   {  TranslateMessage(&msg);
      DispatchMessage(&msg);
      switch(msg.message)
      {  case WM_LBUTTONDOWN:
            mouse_x = LOWORD(msg.lParam);
            mouse_y = HIWORD(msg.lParam);
            p = Point(disp_to_user_x(mouse_x), disp_to_user_y(mouse_y));
            MoveWindow (child_hwnd, 0, 0, 0, 0, true);
            MoveWindow (edit_hwnd, 0, 0, 0, 0, true);
            return p;

         case WM_MOUSEMOVE:
            mouse_x = LOWORD(msg.lParam);
            mouse_y = HIWORD(msg.lParam);
            sprintf(buf, "(%f,%f)",disp_to_user_x(mouse_x),disp_to_user_y(mouse_y)); 
            SendMessage(edit_hwnd, WM_SETTEXT, 0, (LPARAM)buf);
            break;
      }
   }

   return p;
}

/***************************************************************************/

#define main ccc_win_main


