#ifndef CCC_MSW_H
#define CCC_MSW_H

/****************************************************************************
** COPYRIGHT (C):    1994..1998 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++ 2E
** FILE:             ccc_msw.h
** PURPOSE:          Graphics window class (Microsoft Windows version)
** VERSION           1.0
** PROGRAMMER:       Cay Horstmann (CSH)
**                   Charles Miller (CBM)
**                   Darrel D. Cherry (DDC)
**                   Jiaoyang Zhou (JYZ)
**                   Geof Pawlicki (GFP)
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
****************************************************************************/

#line 21 "ccc_msw.cpp"


#ifdef CCC_USE_OLD_CLIBS
   #include <math.h>
   #include <string.h>
   #include <stdlib.h>
   #include <stdio.h>
#else
   #include <cmath>
   #include <cstring>
   #include <cstdlib>
   #include <cstdio>
#endif

#include <windows.h>

// avoid conflicts with Windows function names
#define Polygon CCC_WIN_Polygon
#define Ellipse CCC_WIN_Ellipse
#define Rectangle CCC_WIN_Rectangle


#line 56 "ccc_msw.cpp"


class GraphicBitmap
{
public:
   GraphicBitmap();
   void init(HWND hwnd);
   void close() const;
   void repaint(PAINTSTRUCT&) const;
   void clear() const;
   HDC hdc() const;

private:
   HDC _hdc; // a long-lived device context
   HBITMAP _hbm, _hbm_saved; // handles to the created bitmap objects
   int _disp_xmax; // width of the bitmap (in bits)
   int _disp_ymax; // height of the bitmap (in bits)
};

/*-------------------------------------------------------------------------*/

class GraphicWindow
// PURPOSE: Sends graphics primitives to a display window
{
public:
   GraphicWindow();
   void open(HWND hwnd, HDC mainwin_hdc);
   void coord(double xmin, double ymin, double xmax, double ymax);
   void clear();

   GraphicWindow& operator<<(Point p);
   GraphicWindow& operator<<(Circle c);
   GraphicWindow& operator<<(Line s);
   GraphicWindow& operator<<(Message t);

   string get_string(string out_string);
   int get_int(const string& out_string) 
   {  return atoi(get_string(out_string).c_str()); 
   }
   double get_double(const string& out_string) 
   {  return atof(get_string(out_string).c_str()); 
   }
   Point get_mouse(string out_string);

private:
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   double disp_to_user_x(int x) const;
   double disp_to_user_y(int y) const;

   void statusline_prompt(string s);

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void ellipse(double x, double y, double ra, double rb);

   void text(string t, double x, double y);

   double _user_xmin; // the user's window's logical window coords
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;
   int _disp_xmax; // the physical window dimension (in pixels)
   int _disp_ymax;

   GraphicBitmap& _bm; // the bitmap to draw on
   HDC _hdc; // device context to the display
   int _mlength; // length of text extent for message child window
   int _elength; // length of text extent for edit child window
};


#line 137 "ccc_msw.cpp"
extern GraphicWindow cwin;

#endif
