/****************************************************************************
** COPYRIGHT (C):    1995 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++ 2E
** FILE:             ccc_shap.cpp
** PURPOSE:          defines shapes for graphics
** VERSION           2.0
** PROGRAMMER:       Cay Horstmann (CSH)
**                   Jiaoyang Zhou
** NOTE TO STUDENTS: Once you mastered chapter 8, you should be able
** to understand this file. Just ignore the EXPORT directives, though.
** They are used to automatically extract header files from source files.
****************************************************************************/

#define EXPORT

#ifndef EXPORT

#ifndef CCC_ANSI_H
   #include <string>
   using namespace std;
#endif

#ifdef CCC_USE_OLD_CLIBS
   #include <stdio.h>
#else
   #include <cstdio>
#endif

#endif
  // EXPORT

#include "ccc_shap.h"

/*-------------------------------------------------------------------------*/

#ifndef EXPORT

class Point
{
public:
   Point();
   Point(double x1, double y1);
   double get_x() const;
   double get_y() const;
   void move(double dx, double dy);
private:
   double x;
   double y;
};

/*-------------------------------------------------------------------------*/

class Circle
{
public:
   Circle();
   Circle(Point p, double r);
   Point get_center() const;
   double get_radius() const;
   void move(double dx, double dy);
private:
   Point center;
   double radius;
};

/*-------------------------------------------------------------------------*/

class Line
{
public:
   Line();
   Line(Point p1, Point p2);
   Point get_start() const;
   Point get_end() const;
   void move(double dx, double dy);

private:
   Point from;
   Point to;
};

/*-------------------------------------------------------------------------*/

class Message
{
public:
   Message();
   Message(Point s, double x);
   Message(Point s, const string& m);
   Point get_start() const;
   string get_message() const;
   void move(double dx, double dy);
private:
   Point start;
   string message;
};

#endif

/*-------------------------------------------------------------------------*/

Point::Point()
{  x = 0.0;
   y = 0.0;
}

Point::Point(double x1, double y1)
{  x = x1;
   y = y1;
}

double Point::get_x() const
{  return x;
}

double Point::get_y() const
{  return y;
}

void Point::move(double dx, double dy)
{  x += dx;
   y += dy;
}

/*-------------------------------------------------------------------------*/

Circle::Circle()
{  radius = 0.0;
}

Circle::Circle(Point p, double r)
{  center = p;
   radius = r;
}

Point Circle::get_center() const
{  return center;
}

double Circle::get_radius() const
{  return radius;
}

void Circle::move(double dx, double dy)
{  center.move(dx, dy);
}

/*-------------------------------------------------------------------------*/

Line::Line()
{
}

Line::Line(Point p1, Point p2)
{  from = p1;
   to = p2;
}

Point Line::get_start() const
{  return from;
}

Point Line::get_end() const
{  return to;
}

void Line::move(double dx, double dy)
{  from.move(dx, dy);
   to.move(dx, dy);
}

/*-------------------------------------------------------------------------*/

Message::Message()
{}

Message::Message(Point s, const string& m)
{  start = s;
   message = m;
}

Message::Message(Point s, double x)
{  start = s;
   char buf[20];
   sprintf(buf, "%g", x);
   message = buf;
}


Point Message::get_start() const
{  return start;
}

string Message::get_message() const
{  return message;
}

void Message::move(double dx, double dy)
{  start.move(dx, dy);
}

