#ifndef CCC_STRG_H
#define CCC_STRG_H

/****************************************************************************
** COPYRIGHT (C):    1992...1998 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++ 2E
** FILE:             ccc_strg.h
** PURPOSE:          a subset of the ANSI string class with bounds checking
** VERSION           2.0
** PROGRAMMERS:      Cay Horstmann (CSH)
**                   Darrel D. Cherry (DDC)
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
****************************************************************************/

#line 18 "ccc_strg.cpp"

   #ifdef CCC_USE_OLD_CLIBS
      #include <string.h>
      #include <ctype.h>
      #include <assert.h>
   #else
      #include <cstring>
      #include <cctype>
      #include <cassert>
   #endif

   #ifdef CCC_USE_OLD_STREAMS
      #include <iostream.h>
   #else
      #include <iostream>
   #endif

#line 40 "ccc_strg.cpp"


#define CCC_CHECK_BOUNDS(I) (check_bounds(__FILE__, __LINE__, I))

class string
{
public:
   string();                       // constructs an empty string
   string(const char s[]);
   string(char ch);

   int length() const;
   string substr(int from, int n) const;

   char& operator[](int i);
   char operator[](int i) const;

   const string& operator+=(const string& s);
   const string& operator+=(const char s[]);
   const string& operator+=(char ch);

   string operator+(const string&) const;
   string operator+(const char b[]) const;
   string operator+(char) const;

   friend string operator+(const char a[], const string& b);
   friend string operator+(char ch, const string& b);

   friend ostream& operator<<(ostream& os, const string& s);
   friend istream& operator>>(istream& os, string& s);

   const char* c_str() const { return _str ? _str : ""; }
   int compare(const string&) const;
   int compare(const char s[]) const;

   string(const string&);
   const string& operator=(const string&);
   ~string();

private:
   void check_bounds(const char* f, int l, int i) const;

   string(size_t len, const char s[], size_t slen);
   string(int);
   /* left undefined--to avoid accidental conversion int -> char -> string
   */
   void detach();
   void unique();

   char* _str;
};

/*-------------------------------------------------------------------------*/

class StringHeader
/* PURPOSE:    The header block with reference count and available space
   REMARKS:    An instance of this structure is BELOW each string
               that is allocated on the collectible heap
*/
{
public:
   int unique() { return _rc == 1; }
   void incref() { ++_rc; }
   int decref() { return --_rc; }

   size_t length() const { return _length; }

   static char* alloc(const char[]);
   static char* alloc(int, const char[], int);
   static void free(char*);

   static StringHeader* header(const char* p) { return (StringHeader*)p - 1; }

private:
   char* start() const { return (char*)(this+1); }

   unsigned _rc;                       // reference count
   size_t _length;

friend class string;
};

/* FUNCTIONS ***************************************************************/

inline int string::length() const
/* RETURNS:       the length of the string
*/
{  return _str ? StringHeader::header(_str)->_length : 0;
}

/*...........................................................................*/

inline int operator==(const string& a, const string& b)
{ return a.compare(b) == 0; }
inline int operator!=(const string& a, const string& b)
{ return a.compare(b) != 0; }
inline int operator<(const string& a, const string& b)
{ return a.compare(b) < 0; }
inline int operator<=(const string& a, const string& b)
{ return a.compare(b) <= 0; }
inline int operator>(const string& a, const string& b)
{ return a.compare(b) > 0; }
inline int operator>=(const string& a, const string& b)
{ return a.compare(b) >= 0; }
 
inline int operator==(const string& a, const char b[])
{ return a.compare(b) == 0; }
inline int operator!=(const string& a, const char b[])
{ return a.compare(b) != 0; }
inline int operator<(const string& a, const char b[])
{ return a.compare(b) < 0; }
inline int operator<=(const string& a, const char b[])
{ return a.compare(b) <= 0; }
inline int operator>(const string& a, const char b[])
{ return a.compare(b) > 0; }
inline int operator>=(const string& a, const char b[])
 { return a.compare(b) >= 0; }

inline int operator==(const char a[], const string& b)
{ return b.compare(a) == 0; }
inline int operator!=(const char a[], const string& b)
{ return b.compare(a) != 0; }
inline int operator<(const char a[], const string& b)
 { return b.compare(a) > 0; }
inline int operator<=(const char a[], const string& b)
 { return b.compare(a) >= 0; }
inline int operator>(const char a[], const string& b)
 { return b.compare(a) < 0; }
inline int operator>=(const char a[], const string& b)
{ return b.compare(a) <= 0; }


#line 384 "ccc_strg.cpp"
extern string operator+(const char a[], const string& b);

#line 398 "ccc_strg.cpp"
extern string operator+(char ch, const string& b);

#line 410 "ccc_strg.cpp"
extern ostream& operator<<(ostream& os, const string& s);

#line 422 "ccc_strg.cpp"
extern istream& operator>>(istream& is, string& s);

#line 545 "ccc_strg.cpp"
extern void getline(istream& is, string& r);

#endif
