#ifndef CCC_X11_H
#define CCC_X11_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
**
** NOTE TO STUDENTS: Do not attempt to study the contents of this file. You
** can, and should, use the services provided in this file without knowing
** the highly technical details of the implementation.
**
** PROJECT:          Computing Concepts with C++ Essentials
** FILE:             ccc_x11.h
** PURPOSE:          Graphics context class (X-Window's version)
** VERSION           2.0
** COMPILER:         GNU g++
** TARGET:           486, Linux
** PROGRAMMER:       Cay Horstmann (CSH)
**                   Darrel D. Cherry (DDC)
**                   Charles Miller (CBM)
**                   Geof Pawlicki (GFP)
** START DATE:       6-5-94  (CBM)
** UPDATE HISTORY:   8-2-95  (DDC)
**                   9-25-95 (DDC) - fixed repainting in read string/point
**                   6-1-97  (CSH) - removed font and colors
****************************************************************************/

#line 28 "ccc_x11.cpp"


#ifdef CCC_USE_OLD_CLIBS
   #include <math.h>
   #include <stdlib.h>
#else
   #include <cmath>
   #include <cstdlib>
#endif

#define Time X11_TIME    // so that Time class doesn't conflict with #typedef
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/Xos.h>
#undef Time


#line 76 "ccc_x11.cpp"


class GraphicBitmap
/* PURPOSE: Provides pixmap memory for screen redraws
   NOTES:  Make sure global xgc has been initialized.
*/
{
public:
   GraphicBitmap();
   void init();
   void close();
   void repaint() const;
   void clear_bitmap(unsigned long color) const;
   Pixmap id() const;
private:
   Pixmap _ppm;
};


/**************************************************************************/

class GraphicWindow
/* PURPOSE: Sends graphics primitives to a display window
*/
{
public:
   GraphicWindow();
   GraphicWindow(double x1, double y1, double x2, double y2);
   void coord(double xmin, double ymin, double xmax, double ymax);
   void close();
   void clear() const;
   void common_init();

   /*** overloaded operators ***/

   GraphicWindow& operator<<(Point p);
   GraphicWindow& operator<<(Circle c);
   GraphicWindow& operator<<(Line l);
   GraphicWindow& operator<<(Message m);

   /*** User prompted input  ***/

   string get_string(string s);
   int get_int(const string& out_string) 
   {  return atoi(get_string(out_string).c_str()); 
   }
   double get_double(const string& out_string) 
   {  return atof(get_string(out_string).c_str()); 
   }
   Point get_mouse(string outstr);

private:

   /***  I/O  ***/     

   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   int user_to_disp_dx(double x) const;
   int user_to_disp_dy(double y) const;
   double disp_to_user_x(int x) const;
   double disp_to_user_y(int y) const;

   void statusline_prompt(string s);

   /***  Shapes  ***/     

   void point(double x, double y) const;
   void line(double xfrom, double yfrom, double xto, double yto) const;
   void ellipse(double x, double y, double ra, double rb) const;
   void text(const char t[], double x, double y) const;

   int put_string(const char[], int, int);

   string _display_string;    // string displayed to user at top

   double _user_xmin;            // the user's window's logical window coords
   double _user_ymin;            // min is upper left   
   double _user_xmax;            // max lower right 
   double _user_ymax;
   int _disp_xmax;               // the physical window dimension (in pixels)
   int _disp_ymax;

   XFontStruct* _fontinfo_ptr;    // points to font info for all fonts used
};


#line 166 "ccc_x11.cpp"
extern GraphicWindow cwin;

#endif
