/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             database.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>

using namespace std;

   #endif

#include "ccc_empl.cpp"

const int NEWLINE_LENGTH = 2; /* or 1 on Unix */
const int RECORD_SIZE = 30 + 10 + NEWLINE_LENGTH;

double string_to_double(string s)
/* PURPOSE:  Convert a string to a floating-point value, e.g. "3.14" -> 3.14
   RECEIVES: s - a string representing a floating-point value
   RETURNS:  the equivalent floating-point value
*/   
{  istringstream instr(s);
   double x;
   instr >> x;
   return x;
}

void raise_salary(Employee& e, double percent)
/* PURPOSE:  Raise an employee salary 
   RECEIVES: e - employee receiving raise
             by - the percentage of the raise
*/
{  double new_salary = e.get_salary() * (1 + percent / 100);
   e.set_salary(new_salary);
}

void read_employee(Employee& e, fstream& fs)
/* PURPOSE:  Read an employee record from a file
   RECEIVES: e - filled with the employee
             fs - the file stream to read from
*/
{  string line;
   getline(fs, line);
   if (fs.fail()) return;
   string name = line.substr(0, 30);
   double salary = string_to_double(line.substr(30, 10));
   e = Employee(name, salary);
}

void write_employee(Employee e, fstream& fs)
/* PURPOSE:  Write an employee record to a file
   RECEIVES: e - the employee record to write
             fs - the file stream to write to
*/
{  fs << e.get_name()
      << setw(10 + (30 - e.get_name().length()))
      << fixed << setprecision(2)
      << e.get_salary()
      << "\n";
}

int main()
{  cout << "Please enter the data file name: ";
   string filename;
   cin >> filename;
   fstream fs;
   fs.open(filename.c_str());
   fs.seekg(0, ios::end); /* go to end of file */
   int nrecord = fs.tellg() / RECORD_SIZE;

   cout << "Please enter the record to update: (0 - "
      << nrecord - 1 << ") ";
   int pos;
   cin >> pos;

   const double SALARY_CHANGE = 5.0;

   Employee e;
   fs.seekg(pos * RECORD_SIZE, ios::beg);
   read_employee(e, fs);
   raise_salary(e, SALARY_CHANGE);
   fs.seekp(pos * RECORD_SIZE, ios::beg);
   write_employee(e, fs);

   fs.close();
   return 0;
}
