/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             readdata.cpp
****************************************************************************/

#include "ccc_win.cpp"

/* PURPOSE:  Read a series of floating-point data from a stream
   RECEIVES: v - the vector to hold the input data
             in - the input stream from which to read the data
*/

void read_data(vector<double>& v, ifstream& in)
/* PURPOSE:  read data into a vector
   RECEIVES: v - the vector to fill
             in - the input stream to read from
*/            
{  double s;
   while (in >> s)
      v.push_back(s);
}

double maximum(vector<double> v)
/* PURPOSE:  compute the maximum value in a vector
   RECEIVES: a - the vector
*/             
{  if (v.size() == 0) return 0;
   double highest = v[0];
   int i;
   for (i = 1; i < v.size(); i++)
      if (v[i] > highest)
         highest = v[i];
   return highest;
}

void plot_data(vector<double> v)
/* PURPOSE:  Plot the data in a vector as a bar chart
   RECEIVES: v - the data to plot
*/
{  double highest = maximum(v);
   cwin.coord(0, 0, highest, v.size());
   int i;
   for (i = 0; i < v.size(); i++)
   {  Point left(0, i);
      Point right(v[i], i);
      Line bar(left, right);
      Message label(left, v[i]);
      cwin << bar << label;
   }
}

int main()
{  string filename
      = cwin.get_string("Please enter the data file name: ");
   ifstream infile;
   infile.open(filename.c_str());
   vector<double> data;
   read_data(data, infile);
   infile.close();
   plot_data(data);
   return 0;
}
