/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             input.cpp
****************************************************************************/

   #include "ccc_ansi.h"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <string>
#include <cctype>

using namespace std;

   #endif

#include "error.h"
#include "input.h"

bool is_operator(string s)
/* PURPOSE:  Test if a token is an operator
   RECEIVES: s - the token
   RETURNS:  true if s is one of: + - * / ^
*/
{  return s == "+" or s == "-" or s == "*" or s == "/" or s == "^";
}

void skip_whitespace()
/* PURPOSE:  Skip all white space in standard input
*/
{  char ch;
   do
   {  cin.get(ch);
      if (cin.fail()) return;
   } while (isspace(ch));
#ifndef CCC_USE_OLD_STREAMS
   cin.unget();
#else
   cin.putback(ch);
      /* used if your compiler doesn't support the ANSI unget function */
#endif
}

string next_number()
/* PURPOSE:  read the next number from standard input
   RETURNS:  the next number as a string
*/
{  string r;
   bool more = true;
   while (more)
   {  char ch;
      cin.get(ch);
      if (cin.fail())
         more = false;
      else if (isdigit(ch))
         r = r + ch;
      else
      {
#ifndef CCC_USE_OLD_STREAMS
         cin.unget();
#else
         cin.putback(ch);
            /* used if your compiler doesn't support the ANSI unget function */
#endif
         more = false;
      }
	}
   return r;
}

string next_token()
/* PURPOSE:  read the next token from standard input
   RETURNS:  the next token
*/
{  skip_whitespace();
   char ch;
   cin.get(ch);
   if (cin.fail()) return "";   
   if (isdigit(ch))
   {
#ifndef CCC_USE_OLD_STREAMS
      cin.unget();
#else
      cin.putback(ch);
         /* used if your compiler doesn't support the ANSI unget function */
#endif
      return next_number();
   }
	else
   {  string token;
      token = token + ch;
      if (is_operator(token) or token == "(" or token == ")"
            or token == "=")
         return token;
      else
      {  string message = "Unexpected input ";
         error(message + ch);
         return "";
      }
   }
}
