/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             mergsort.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <vector>
#include <cstdlib>
#include <ctime>

using namespace std;

   #endif

void merge(vector<int>& a, int from, int mid, int to)
/* PURPOSE:  merge two adjacent ranges in a vector
   RECEIVES: a - the vector with the elements to merge
             from, mid - start and end of the first range
             to - end of the second range
*/
{  int n = to - from + 1; /* size of the range to be merged */
   /* merge both halves into a temporary vector b */
   vector<int> b(n);

   int i1 = from;
      /* next element to consider in the first half */
   int i2 = mid + 1;
      /* next element to consider in the second half */
   int j = 0; /* next open position in b */

   /* as long as neither i1 nor i2 past the end, move the smaller
      element into b
   */
   while (i1 <= mid && i2 <= to)
   {  if (a[i1] < a[i2])
      {  b[j] = a[i1];
         i1++;
      }
      else
      {  b[j] = a[i2];
         i2++;
      }
      j++;
   }

   /* note that only one of the two while loops below is executed
   */

   /* copy any remaining entries of the first half */
   while (i1 <= mid)
   {  b[j] = a[i1];
      i1++;
      j++;
   }
   /* copy any remaining entries of the second half */
   while (i2 <= to)
   {  b[j] = a[i2];
      i2++;
      j++;
   }

   /* copy back from the temporary vector */
   for (j = 0; j < n; j++)
      a[from + j] = b[j];
}

void merge_sort(vector<int>& a, int from, int to)
/* PURPOSE:  sort the elements in a range of a vector
   RECEIVES: a - the vector with the elements to sort
             from, to - start and end of the range
*/
{  if (from == to) return;
   int mid = (from + to) / 2;
    /* sort the first and the second half */
   merge_sort(a, from, mid);
   merge_sort(a, mid + 1, to);
   merge(a, from, mid, to);
}

void print(vector<int> a)
/* PURPOSE:  print all elements in a vector
   RECEIVES: a - the vector to print
*/
{  int i;
   for (i = 0; i < a.size(); i++)
      cout << a[i] << " ";
   cout << "\n";
}

void rand_seed()
/* PURPOSE: Set the seed of the random number generator
*/
{  int seed = static_cast<int>(time(0));
   srand(seed);
}

int rand_int(int a, int b)
/* PURPOSE:  Return a random integer in a range
   RECEIVES: a - the bottom of the range
             b - the top of the range
   RETURNS:  A random number x, a <= x and x <= b
*/
{  return a + rand() % (b - a + 1);
}

int main()
{  rand_seed();
   vector<int> v(20);
   int i;
   for (i = 0; i < v.size(); i++)
      v[i] = rand_int(1, 100);
   print(v);
   merge_sort(v, 0, v.size() - 1);
   print(v);
   return 0;
}
