/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             sorttime.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <vector>
#include <cstdlib>
#include <ctime>

using namespace std;

   #endif

#include "ccc_time.cpp"

void swap(int& x, int& y)
/* PURPOSE:  swap two integers
   RECEIVES: x, y - the integers to swap
*/
{  int temp = x;
   x = y;
   y = temp;
}

int min_position(vector<int>& a, int from, int to)
/* PURPOSE:  get the position of the smallest element in a vector range
   RECEIVES: a - a vector
             from, to - a range
   RETURNS:  the position of the smallest element in a[from]...a[to]
*/
{  int min_pos = from;
   int i;
   for (i = from + 1; i <= to; i++)
      if (a[i] < a[min_pos]) min_pos = i;
   return min_pos;
}

void selection_sort(vector<int>& a)
/* PURPOSE:  sort a vector using the selection sort algorithm
   RECEIVES: a - the vector to sort
*/
{  int next; /* the next position to be set to the minimum */

   for (next = 0; next < a.size() - 1; next++)
   {  /* find the position of the minimum */
      int min_pos = min_position(a, next, a.size() - 1);
      if (min_pos != next)
         swap(a[min_pos], a[next]);
   }
}

void rand_seed()
/* PURPOSE: Set the seed of the random number generator
*/
{  int seed = static_cast<int>(time(0));
   srand(seed);
}

int rand_int(int a, int b)
/* PURPOSE:  Return a random integer in a range
   RECEIVES: a - the bottom of the range
             b - the top of the range
   RETURNS:  A random number x, a <= x and x <= b
*/
{  return a + rand() % (b - a + 1);
}

int main()
{  rand_seed();
   cout << "Enter vector size: ";
   int n;
   cin >> n;
   vector<int> v(n);
   int i;
   for (i = 0; i < v.size(); i++)
      v[i] = rand_int(1, 100);
   Time before;
   selection_sort(v);
   Time after;

   cout << "Elapsed time = " << after.seconds_from(before)
      << " seconds\n";
   return 0;
}
