/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             list2.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <string>
#include <iostream>
#include <cassert>

using namespace std;

   #endif

template<typename T>
class List;

template<typename T>
class Iterator;

template<typename T>
class Link
{
public:
   Link(T s);
private:
   T data;
   Link<T>* previous;
   Link<T>* next;
friend class List<T>;
friend class Iterator<T>;
};

template<typename T>
class List
{
public:
   List();
   void push_back(T s);
   void insert(Iterator<T> iter, T s);
   void erase(Iterator<T> iter);
   Iterator<T> begin();
   Iterator<T> end();
private:
   Link<T>* first;
   Link<T>* last;
};

template<typename T>
class Iterator
{
public:
   Iterator();
   T operator*() const;
   void operator++(int dummy);
   void operator--(int dummy);
   bool operator!=(Iterator<T> b) const;
private:
   Link<T>* position;
   Link<T>* last;
friend class List<T>;
};

template<typename T>
Link<T>::Link(T s)
/* RECEIVES:  s - the data to store in this link
*/
{  data = s;
   previous = NULL;
   next = NULL;
}

template<typename T>
List<T>::List()
{  first = NULL;
   last = NULL;
}

template<typename T>
void List<T>::push_back(T s)
/* PURPOSE:  append an element to the list
   RECEIVES: s - the value to append
*/
{  Link<T>* newlink = new Link<T>(s);
   if (last == NULL) /* list is empty */
   {  first = newlink;
      last = newlink;
   }
   else
   {  newlink->previous = last;
      last->next = newlink;
      last = newlink;
   }
}

template<typename T>
void List<T>::insert(Iterator<T> iter, T s)
/* PURPOSE:  insert an element into the list
   RECEIVES: iter - the position before which to insert
             s - the value to append
*/
{  if (iter.position == NULL)
   {  push_back(s);
      return;
   }

   Link<T>* after = iter.position;
   Link<T>* before = after->previous;
   Link<T>* newlink = new Link<T>(s);
   newlink->previous = before;
   newlink->next = after;
   after->previous = newlink;
   if (before == NULL) /* insert at beginning */
      first = newlink;
   else
      before->next = newlink;
}

template<typename T>
void List<T>::erase(Iterator<T> iter)
/* PURPOSE:  remove an element from the list
   RECEIVES: iter - the position to remove
*/
{  assert(iter.position != NULL);
   Link<T>* remove = iter.position;
   Link<T>* before = remove->previous;
   Link<T>* after = remove->next;
   if (remove == first)
      first = after;
   else
      before->next = after;
   if (remove == last)
      last = before;
   else
      after->previous = before;
   iter.position = after;
   delete remove;
}

template<typename T>
Iterator<T> List<T>::begin()
/* PURPOSE:  get the beginning position of the list
   RETURNS:  an iterator pointing to the beginning of the list
*/
{  Iterator<T> iter;
   iter.position = first;
   iter.last = last;
   return iter;
}

template<typename T>
Iterator<T> List<T>::end()
/* PURPOSE:  get the past-the-end position of the list
   RETURNS:  an iterator pointing past the end of the list
*/
{  Iterator<T> iter;
   iter.position = NULL;
   iter.last = last;
   return iter;
}

template<typename T>
Iterator<T>::Iterator()
{  position = NULL;
   last = NULL;
}

template<typename T>
T Iterator<T>::operator*() const
/* PURPOSE:  look up the value at a position
   RETURNS:  the value of the Link<T> to which the iterator points
*/
{  assert(position != NULL);
   return position->data;
}

template<typename T>
void Iterator<T>::operator++(int dummy)
/* PURPOSE:  advance the iterator to the next Link<T>
*/
{  assert(position != NULL);
   position = position->next;
}

template<typename T>
void Iterator<T>::operator--(int dummy)
/* PURPOSE:  move the iterator to the previous Link<T>
*/
{  if (position == NULL)
      position = last;
   else 
      position = position->previous;
   assert(position != NULL);
}

template<typename T>
bool Iterator<T>::operator!=(Iterator<T> b) const
/* PURPOSE:  compare two iterators
   RECEIVES: b - the iterator to compare with this iterator
   RETURNS:  true if this iterator and b are not equal
*/
{  return position != b.position;
}

int main()
{  List<string> staff;

   staff.push_back("Cracker, Carl");
   staff.push_back("Hacker, Harry");
   staff.push_back("Lam, Larry");
   staff.push_back("Sandman, Susan");

   /* add a value in fourth place */

   Iterator<string> pos;
   pos = staff.begin();
   pos++;
   pos++;
   pos++;

   staff.insert(pos, "Reindeer, Rudolf");

   /* remove the value in second place */

   pos = staff.begin();
   pos++;

   staff.erase(pos);

   /* print all values */

   for (pos = staff.begin(); pos != staff.end(); pos++)
      cout << *pos << "\n";

   return 0;
}

