/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             buffon.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <ctime>

using namespace std;

   #endif

void rand_seed()
/* PURPOSE: Set the seed of the random number generator
*/
{  int seed = static_cast<int>(time(0));
   srand(seed);
}

double rand_double(double a, double b)
/* PURPOSE:  compute a random floating point number in a range
   RECEIVES: a - the bottom of the range
             b - the top of the range
   RETURNS:  a random floating point number x, a <= x and x <= b
*/
{  return a + (b - a) * rand() * (1.0 / RAND_MAX);
}

double deg2rad(double alpha)
/* PURPOSE:  convert an angle from degree to radian
   RECEIVES: alpha - the angle in degrees
   RETURNS:  the angle in radian
*/
{  const double PI = 3.141592653589793;
   return alpha * PI / 180;
}

int main()
{  int NTRIES = 10000;
   int i;
   int hits = 0;
   rand_seed();
   for (i = 1; i <= NTRIES; i++)
   {  double ylow = rand_double(0, 2);
      double angle = rand_double(0, 180);
      double yhigh = ylow + sin(deg2rad(angle));
      if (yhigh >= 2) hits++;                      
   }
   cout << "Tries / Hits = " << NTRIES * (1.0 / hits) << "\n";
   return 0;
}
