/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             sqroot.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <cmath>

using namespace std;

   #endif

bool not_equal(double x, double y)
/* PURPOSE:  Compare two floating-point numbers
   RECEIVES: x, y - two floating-point numbers
   RETURNS:  true if x is not very close to y
*/
{  const double EPSILON = 1E-14;
   double dmax = fabs(x);
   if (fabs(y) > dmax) dmax = fabs(y);
   return fabs(x - y) > dmax * EPSILON;
}

double squareroot(double a)
/* PURPOSE:  Compute the square root using Heron's formula
   RECEIVES: a - a positive integer
   RETURNS:  the square root of a
*/
{  if (a == 0) return 0;

   double xnew = a;
   double xold;

   do
   {  xold = xnew;
      xnew = (xold + a / xold) / 2;
   }
   while (not_equal(xnew, xold));

   return xnew;
}

int main()
{  cout << "Please enter a number: ";
   double x;
   cin >> x;
   cout << "The square root is " << squareroot(x) << "\n";
   return 0;
}


