/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             table.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <iomanip>
#include <cmath>

using namespace std;

   #endif

double future_value(double initial_balance, double p, int nyear)
/* PURPOSE:  computes the value of an investment with compound
             interest
   RECEIVES: initial_balance - the initial value of the investment
             p - the interest rate in percent
             nyear - the number of years the investment is held
   REMARKS:  Interest is compounded monthly
*/
{  double b = initial_balance * pow(1 + p / 12 / 100, 12 * nyear);
   return b;
}

int main()
{  const double RATE_MIN = 5;
   const double RATE_MAX = 10;
   const double RATE_INCR = 0.5;
   const int YEAR_MIN = 5;
   const int YEAR_MAX = 30;
   const int YEAR_INCR = 5;

   /* print table header */

   cout << " Rate  ";
   int year;
   for (year = YEAR_MIN; year <= YEAR_MAX; year = year + YEAR_INCR)
   {  cout << setw(2) << year << " years  ";
   }
   cout << "\n";

   cout << fixed << setprecision(2);

   double rate;
   double initial_balance = 10000;
   for (rate = RATE_MIN; rate <= RATE_MAX; rate = rate + RATE_INCR)
   {  /* print table row */
       int year;
       cout << setw(5) << rate;
       for (year = YEAR_MIN; year <= YEAR_MAX;
          year = year + YEAR_INCR)
       {  double balance = future_value(initial_balance, rate, year);
          cout << setw(10) << balance;
       }
       cout << "\n";
   }

   return 0;
}
