/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             sqrtest5.cpp
** NOTE TO STUDENTS: This file has been edited to be compatible with older
** compilers. If your compiler fully supports the ANSI/ISO C++
** standard, remove the line #include "ccc_ansi.cpp". You can also remove
** the lines #ifndef CCC_ANSI_H and #endif
****************************************************************************/

   #include "ccc_ansi.cpp"
   #ifndef CCC_ANSI_H

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <ctime>

using namespace std;

   #endif

void rand_seed()
/* PURPOSE: Set the seed of the random number generator
*/
{  int seed = static_cast<int>(time(0));
   srand(seed);
}

double rand_double(double a, double b)
/* PURPOSE:  compute a random floating point number in a range
   RECEIVES: a - the bottom of the range
             b - the top of the range
   RETURNS:  a random floating point number x, a <= x and x <= b
*/
{  return a + (b - a) * rand() * (1.0 / RAND_MAX);
}

bool not_equal(double x, double y)
/* PURPOSE:  Compare two floating-point numbers
   RECEIVES: x, y - two floating-point numbers
   RETURNS:  true if x is not very close to y
*/
{  const double EPSILON = 1E-14;
   double dmax = fabs(x);
   if (fabs(y) > dmax) dmax = fabs(y);
   return fabs(x - y) > dmax * EPSILON;
}

/* Function to be tested */

double squareroot(double a)
/* PURPOSE:  Compute the square root using Heron's formula
   RECEIVES: a - a positive integer
   RETURNS:  the square root of a
*/
{  if (a == 0) return 0;

   double xnew = a;
   double xold;

   do
   {  xold = xnew;
      xnew = (xold + a / xold) / 2;
   }
   while (not_equal(xnew, xold));

   return xnew;
}


/* Test stub */

int main()
{  rand_seed();
   int i;
   for (i = 1; i <= 100; i++)
   {  double x = rand_double(0, 1E6);
      double y = squareroot(x);
      if (not_equal(y, pow(x, 0.5))) 
         cout << "Test failed. ";
      else 
         cout << "Test passed. ";
      cout << "squareroot of " << x << " = " << y << "\n";
   }

   return 0;
}
