/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with C++
** FILE:             cloud.cpp
****************************************************************************/

#include "ccc_win.cpp"

class Cloud
{
public:
   Cloud();
   void add_point(Point p);
   void plot() const;
private:
   vector<Point> dots;
};

void rand_seed()
/* PURPOSE: Set the seed of the random number generator
*/
{  int seed = static_cast<int>(time(0));
   srand(seed);
}

double rand_double(double a, double b)
/* PURPOSE:  compute a random floating point number in a range
   RECEIVES: a - the bottom of the range
             b - the top of the range
   RETURNS:  a random floating point number x, a <= x and x <= b
*/
{  return a + (b - a) * rand() * (1.0 / RAND_MAX);
}

Cloud::Cloud()
{
}

void Cloud::add_point(Point p)
/* PURPOSE:  add a point to the polygon
   RECEIVES: p - the point to add
*/
{  dots.push_back(p);
}

void Cloud::plot() const
/* PURPOSE:  draw the dots in the cloud
*/
{  int i;
   for (i = 0; i < dots.size(); i++)
      cwin << dots[i];
}

int main()
{  rand_seed();
   Cloud random_cloud;
   int i;
   for (i = 1; i <= 100; i++)
      random_cloud.add_point(Point(rand_double(-4, 4),
         rand_double(-4, 4)));
   random_cloud.plot();

   return 0;
}
